
#include "stdafx.h"
#include "ComicMaker.h"

#include <string>  
#include "errno.h"

using namespace std;


const int TempArraySize = 400;

const char MidSeperator = '|';
const char EndSeparator = '\n';

const char inFile[] = "comic_in.htm";
const char outFile[] = "comic_out.htm";
const char IniFileName[] = "comics.ini";
const char UpdateFileName[] = "update.txt";
const char VersionFileName[] = "version.txt";
const char UpdateZipFile[] = "update.zip";

/************************************************/

ComicMaker::ComicMaker(CListBox* inListbox)
{
	theListbox = inListbox;
	return;
}

/************************************************/
ComicMaker::~ComicMaker()
{
	return;
}

/************************************************/
	
void ComicMaker::WriteVersionFile(const int progVersion, const int iniVersion, const int UpdateVersion)
{
	//Write a new version file using the numbers given
	ofstream Writer(VersionFileName);

	Writer << "program";
	Writer << progVersion;
	Writer << "." << endl;

	Writer << "data";
	Writer << iniVersion;
	Writer << "." << endl;

	Writer << "updater";
	Writer << UpdateVersion;
	Writer << "." << endl;


	Writer.close();
}

/************************************************/

void ComicMaker::ParseWebPage(const string &WebPage, string SearchString, string &ComicID)
{
	//WebPage has the entire comic web page.  We are looking for this line (for example):
	//<IMG SRC="/comics/robotman/archive/images/robotman2003479400726.gif" ALT="today's comics strip" ALIGN=TOP BORDER="0">
	//From that, we want the ID number, 2003479400726 in this case.

	//So the basic algorithm -
	//find the string "<IMG SRC="/comics/robotman/archive/images/robotman"
	//Get the numbers after that string and before the following "."
	//Put those numbers in ComicID

	//NEW - this is now more generic.  Rather than have robotman coded into this, it's passed in through
	//SearchString. That lets me use this method for other strips that I have to parse for.

	int Loc = 0, Loc2 = 0;
	Loc = WebPage.find(SearchString);
	if (Loc < 0)
	{
		theListbox->InsertString(0,"Error parsing for the comic id from the web page.");
		//Continue and get the date based items anyway
		ComicID = "NotFound";
		return;
	}
	Loc += SearchString.length();

	//Now Loc is at the start of the ComicID.  Find the end of the comic ID, and
	//put the ComicID into the string waiting for it.

	Loc2 = WebPage.find(".",Loc);
	if (Loc2 < 0)
	{
		theListbox->InsertString(0,"Error parsing for the comic id from the web page.");
		//Continue and get the date based items anyway
		ComicID = "NotFound";
		return;
	}


	ComicID = WebPage.substr(Loc,Loc2 - Loc);

	return;
}

/***************************************************************/
	
bool ComicMaker::ReadInFile(const char *FileToRead, string &OutPage)
{
	//Read the file named by the variable inFile to the string OutPage.
	OutPage = "";

	ifstream Reader(FileToRead);
	if (!Reader)
	{
		string s;
		s = "Can't open input file ";
		s += FileToRead;
		s += " .";
		MessageBox(NULL,s.c_str(),"Error Message",MB_OK);
		return(false);
	}
	
	char inBuf[500];
	//Erase any previous contents of OutPage
	while (Reader.getline(inBuf,500))
	{
		OutPage += inBuf;
		OutPage += "\n";
	}

	return(true);
}

/***************************************************************/

void ComicMaker::ParseSingleToken(string &OutPage, const char* Token, const char* NewVal)
{
	//Walk through OutPage.  Replace each occurrence of Token with NewVal.
	int Loc = 0;

	while ((Loc = OutPage.find(Token, Loc)) >= 0)
	{	
		//Now Loc points to the start of the Token.  Replace it with NewVal.
		OutPage.replace(Loc,strlen(Token),NewVal);
	}	
	return;
}

/***************************************************************/

void ComicMaker::ParseTokens(string &OutPage, time_t theTime)
{

	theTime -= 18000;  //put into US Eastern Time - strips don't pop up at midnight UTC
	struct tm today = *(gmtime(&theTime));
	struct tm WorkingTime = today;

	char temp[TempArraySize];
	DATE_TOKEN currentDateToken;
	URL_TOKEN currentURLToken;
	ifstream Reader(IniFileName);

	//A buffer to hold the date strings
	const int BufSize = 50;
	char CodeBuf[BufSize];

	//Run through the date based codes.  Easy and fast, so do them first.
	//If the token starts with SUNDAY, only replace it on Sundays - otherwise
	//remove it.
	while (GetDateToken(Reader,currentDateToken,temp))
	{
		WorkingTime = today;
		WorkingTime.tm_mday = today.tm_mday - currentDateToken.NumDaysAgo;
		mktime(&WorkingTime);

		//Sunday only tokens
		if (currentDateToken.Name.substr(0,6) == "SUNDAY")
		{
			if (WorkingTime.tm_wday != 0)
			{
				CodeBuf[0] = '\0';
			}
			else
			{
				sprintf(CodeBuf,"%s",currentDateToken.Value.c_str());
			}
			ParseSingleToken(OutPage, currentDateToken.Name.c_str(), CodeBuf);
			continue;
		}

		//Weekday only tokens
		if (currentDateToken.Name.substr(0,7) == "WEEKDAY")
		{
			if (WorkingTime.tm_wday == 0)
			{
				CodeBuf[0] = '\0';
			}
			else
			{
				sprintf(CodeBuf,"%s",currentDateToken.Value.c_str());
			}
			ParseSingleToken(OutPage, currentDateToken.Name.c_str(), CodeBuf);
			continue;
		}

		//a kludge...seems like %u isn't implemented on my compiler.  But %w is.  So fake %u by
		//using %w if needed.
		bool FakingU = false;
		if (currentDateToken.Value == "%u")
		{
			FakingU = true;
			currentDateToken.Value = "%w";
		}
		mktime(&WorkingTime);
		strftime(CodeBuf,BufSize,currentDateToken.Value.c_str(),&WorkingTime);
		if (FakingU && !strncmp(CodeBuf,"0",1))
		{
			sprintf(CodeBuf,"%d",7);
		}
		ParseSingleToken(OutPage, currentDateToken.Name.c_str(), CodeBuf);
	}

	//Now for the hard ones.  These can't be computed.  We
	//actually have to fetch a web page and parse out each token.
	string WebPage, ComicID;


	while (GetURLToken(Reader,currentURLToken,temp))
	{
		//First, see if we need to get this token.  If the token isn't in the
		//in file, don't bother with it.
		if (OutPage.find(currentURLToken.Name) != string::npos)
		{
			//Get the web page
			string s = "About to read web page for ";
			s += currentURLToken.Name;
			theListbox->InsertString(0,s.c_str());
			PageFetcher.GetWebPage(currentURLToken.Domain.c_str(), currentURLToken.Path.c_str(), WebPage);
			//Now, parse through that web page to get the desired token
			ParseWebPage(WebPage,currentURLToken.ParseToken,ComicID);
			ParseSingleToken(OutPage, currentURLToken.Name.c_str(), ComicID.c_str());

			s = "Finished with ";
			s += currentURLToken.Name;
			theListbox->InsertString(0,s.c_str());
		}
		else
		{
			string s = "Skipping unused token ";
			s += currentURLToken.Name;
			theListbox->InsertString(0,s.c_str());
		}
	};

	return;
}

/***************************************************************/

bool ComicMaker::GetDateToken(ifstream &Reader, DATE_TOKEN &token, char* temp)
{
	Reader.getline(temp,TempArraySize,MidSeperator);
	if (strlen(temp) == 0)
		return(false);
	token.Name = temp;
	if (token.Name == "URL")
	{
		Reader.getline(temp,TempArraySize,EndSeparator); // strip the new line character
		return(false);
	}

	//Read the number of days ago
	Reader.getline(temp,TempArraySize,MidSeperator);
	token.NumDaysAgo = atoi(temp);

	Reader.getline(temp,TempArraySize,EndSeparator);
	if (strlen(temp) == 0)
		return(false);
	token.Value = temp;

	return(true);
}

/***************************************************************/

bool ComicMaker::GetURLToken(ifstream &Reader, URL_TOKEN &token, char* temp)
{
	Reader.getline(temp,TempArraySize,MidSeperator);
	if (strlen(temp) == 0)
		return(false);
	token.Name = temp;

	Reader.getline(temp,TempArraySize,MidSeperator);
	if (strlen(temp) == 0)
		return(false);
	token.Domain = temp;

	Reader.getline(temp,TempArraySize,MidSeperator);
	if (strlen(temp) == 0)
		return(false);
	token.Path = temp;

	Reader.getline(temp,TempArraySize,EndSeparator);
	if (strlen(temp) == 0)
		return(false);
	token.ParseToken = temp;

	return(true);
}

/***************************************************************/

void ComicMaker::WriteOutFile(const string &OutPage)
{
	//Read the file named by the variable outFile to the string OutPage.
	ofstream Writer(outFile);
	if (!Writer)
	{
		string s = "Can't open output file ";
		s += outFile;
		s += " .";
		MessageBox(NULL,s.c_str(),"Error Message",MB_OK);
		return;
	}
	
	Writer << OutPage;

	return;
}

/***************************************************************/

bool ComicMaker::CheckForUpdates(bool& iniUpdates, bool& progUpdates, bool& updaterUpdate,
								 int& iniVersion, int& progVersion, int& UpdateVersion)
{
	string PageToRead, Version;
	int FileProgram, Filedata, FileUpdater;
	iniUpdates = false;
	progUpdates = false;
	updaterUpdate = false;

	//Get the version numbers of the stuff on the web
	PageFetcher.GetWebPage("www.evan.org","/Software/comicversion.htm",PageToRead);

	if (PageToRead.empty())
	{
		return(false);
	}


	ParseWebPage(PageToRead,"program",Version);
	progVersion = atoi(Version.c_str());
	ParseWebPage(PageToRead,"data",Version);
	iniVersion = atoi(Version.c_str());
	ParseWebPage(PageToRead,"updater",Version);
	UpdateVersion = atoi(Version.c_str());
	
	//Get the version numbers of the stuff currently being run
	ReadInFile(VersionFileName,PageToRead);
	if (PageToRead.empty())
	{
		return(false);
	}

	ParseWebPage(PageToRead,"program",Version);
	FileProgram = atoi(Version.c_str());
	ParseWebPage(PageToRead,"data",Version);
	Filedata = atoi(Version.c_str());
	ParseWebPage(PageToRead,"updater",Version);
	FileUpdater = atoi(Version.c_str());
	
	//Compare versions. 
	if (progVersion > FileProgram)
	{
		progUpdates = true;
	}
	if (iniVersion > Filedata)
	{
		iniUpdates = true;
	}
	if (UpdateVersion > FileUpdater)
	{
		updaterUpdate = true;
	}
	
	

/*
	//See some updates have been downloaded, but ComicSetup hasn't run to include them.
	ifstream UpdateCheck(UpdateFileName);
	if (UpdateCheck.is_open())
	{
		PageToRead += "<b>You have downloaded updates, but you will need to run ComicSetup to install them!</b><p>";
		UpdateCheck.close();
	}
*/
	return(iniUpdates || progUpdates || updaterUpdate);
}

/***************************************************************/

void ComicMaker::UpdateTheUpdater(const int iniVersion, const int progVersion, const int UpdateVersion)
{
	if (AfxMessageBox("The updater program needs to be updated.  Press \"YES\" to update it now or \"NO\" to leave it alone.",
		MB_YESNO) == IDYES)
	{
		std::string FileName = "ComicUpdater.exe";
		std::string WebFilePath;
		WebFilePath = "/Software/files/";
		WebFilePath += FileName;
		
		PageFetcher.WriteWebPage("www.evan.org",WebFilePath.c_str(),"temp.file");
		remove(FileName.c_str());
		rename("temp.file",FileName.c_str());


		//Write a new version file
		WriteVersionFile(progVersion, iniVersion, UpdateVersion);
	}

	return;
}

/***************************************************************/

//returns true if updates area available
bool ComicMaker::Runner(void)
{
	string OutPage;

	//Now read in the in file.
	if (!ReadInFile(inFile, OutPage))
	{
		MessageBox(NULL,"Can't find the input file.  Make sure you ran the comic setup program to choose your strips before running this program to download them.","Error",MB_OK);
		return(false);
	}

	//What's today's date?
	time_t theTime;

	_tzset();  //set for local time zone

	time(&theTime);
	
	//If it's Monday, check for updates on the web.
	tm MondayCheck;
	MondayCheck = *localtime(&theTime);

	bool updates = false;
	bool updatesToRun = false;
	string UpdateInfo = "";

	bool ini, prog, upd;	
	if (MondayCheck.tm_wday == 1)
	{
		int prog_version,ini_version,updater_version;
		updates = CheckForUpdates(ini, prog, upd, ini_version, prog_version, updater_version);
		if (upd)
		{
			//The updating program itself needs updating.  Get permission to update it and then do it.
			UpdateTheUpdater(ini_version, prog_version, updater_version);
		}
		
		if (ini || prog)
		{
			UpdateInfo += "New updates are available.  A update program should launch automatically, if not you can either launch it from the start menu or go to <a href = \"http://www.evan.org/Software/Comics.html\">this web page</a> to get the downloads yourself.  If you do that, don't forget to run Comic Setup afterwards to incorporate the updates.<p>";
		}
	}
	
	

	//See some updates have been downloaded, but ComicSetup hasn't run to include them.
	ifstream UpdateCheck(UpdateFileName);
	if (UpdateCheck.is_open())
	{
		UpdateInfo += "<b>You have downloaded updates, but you need to run ComicSetup to install them!</b><p>";
		UpdateCheck.close();
		updatesToRun = true;
	}

	if (ini || prog || updatesToRun)
	{
		int Loc = 0;
		Loc = OutPage.find("<body>");
		if (Loc > 0)
		{
			Loc += 6;
			
			OutPage.insert(Loc,UpdateInfo);
		}
	}
	

	//Adjust OutPage to the desired form.
	ParseTokens(OutPage, theTime);

	//Write it to file.
	WriteOutFile(OutPage);

	
	return(updates);
}
